@@ -1,6 +1,6 @@
 /*
     db_dump.c - Routines for reading dBase III files
-    Copyright (c) 1995  Martin Schulze <Martin.Schulze@Infodrom.North.DE>
+    Copyright (c) 1995,96,2003,6  Martin Schulze <joey@infodrom.org>
 
     This file is part of the dbview package, a viewer for dBase II files.
 
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    along with this program;  if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */
 
 
@@ -24,7 +24,9 @@
     Most of the code in this file comes from Greg Twaites anno 87. I
     only took the file and wrote a program around it. I enclude the
     whole header. I have obtained this file from a free software
-    archive, namely nic.funet.fi.
+    archive, namely nic.funet.fi.  It is supposed to be placed in the
+    public domain.  Hence, adding GPL code around and distributinng
+    the entire file under the GPL should be fine.
 
       ftp://nic.funet.fi/pub/msdos/languages/c/dbase.c
 */
@@ -46,6 +48,15 @@
  *	Modified on some places based on corrections from Vladimir
  *	Michl <Vladimir.Michl@upol.cz>
  *
+ * Mon Sep 25 17:09:46 2006: Gergely Szasz <szaszg@hu.inter.net>
+ *	Added --deleted|-D to display deleted records as well.
+ *
+ * Mon Oct 16 18:34:23 CEST 2006: Joey Schulze <joey@infodrom.org>
+ *	Improved error handling.  Improved handling of differences between
+ *	dBase III and dBase III+ files.
+ *
+ * Mon Nov 20 16:58:35 CET 2006: Joey Schulze <joey@infodrom.org>
+ *	Adjusted type handling and proper casts.
  */
  
 #include "db_dump.h"
@@ -57,29 +68,68 @@
 #include <malloc.h>
 #include <ctype.h>
 #include <string.h>
+#include <endian.h>
+#include <byteswap.h>
  
+#define TRUE	1
+#define FALSE	0
+
 DBASE_HEAD  dbhead={0};
 FLD_LIST    *db_fld_root=0;
 char        *Buffer;
 char        buf_work[255];
-int         dbfile;
+FILE        *dbfile;
+
+/*----------------------------------------------------helpers------------*/
+#if BYTE_ORDER == BIG_ENDIAN
+size_t
+db3_fread (void *ptr, size_t size, size_t nmemb, FILE *stream)
+{
+  size_t ret;
+  size_t i;
+  uint16_t *p = (uint16_t *) ptr;
+  uint32_t *pp = (uint32_t *) ptr;
+
+  ret = fread(ptr, size, nmemb, stream);
+
+  switch (size)
+    {
+      case 2:
+	for (i = 0; i < ret; i++)
+	  p[i] = bswap_16(p[i]);
+	break;
+      case 4:
+	for (i = 0; i < ret; i++)
+	  pp[i] = bswap_32(pp[i]);
+	break;
+      default:
+	break;
+    }
+
+  return ret;
+}
+
+#define fread db3_fread
+#endif
  
 /*------------------------------------------------------------code-------*/
-void
+int
 db3_process(dbfn, flags, delim)
 char    *dbfn;
 int	flags;
 char	delim;
 {
-    dbfile=open(dbfn,O_RDONLY);
-    if (dbfile == -1) {
+    dbfile=fopen(dbfn,"r");
+    if (dbfile == NULL) {
         printf("Unable to open file `%s'\n", dbfn);
-	return;
+	return FALSE;
     }
-    db3_read_dic(flags);
-    if ( ! (flags & DB_FL_OMIT))
+    if (db3_read_dic(flags)) {
+      if ( ! (flags & DB_FL_OMIT))
         db3_print_recs(dbhead.count, flags, delim);
-    close(dbfile);
+    }
+  fclose(dbfile);
+  return TRUE;
 }
  
 
@@ -116,7 +166,7 @@ char	*var;
 {
     char *c=var;
 
-    for (c+=strlen(var); isspace (*c); c--);
+    for (c+=strlen(var)-1; isspace (*c); c--);
 
     *(++c) = '\0';
     return var;
@@ -135,9 +185,10 @@ void
 db3_cvt_fld (fld)
 DBASE_FIELD	*fld;
 {
-    char *c;
-    char first=1;
+    uint8_t *c;
+    uint8_t first;
 
+    first = 1;
     for (c=fld->name; *c; c++)
 	if (!first) {
 	    if (isupper(*c))
@@ -151,7 +202,32 @@ DBASE_FIELD	*fld;
 	    first = 0;
 }
 
- 
+
+void
+db3_read_head(dbh)
+DBASE_HEAD *dbh;
+{
+  fread(&dbh->version, 1, 1, dbfile);
+  fread(dbh->l_update, 1, 3, dbfile);
+  fread(&dbh->count, 4, 1, dbfile);
+  fread(&dbh->header, 2, 1, dbfile);
+  fread(&dbh->lrecl, 2, 1, dbfile);
+  fread(dbh->reserv, 1, 20, dbfile);
+}
+
+void
+db3_read_field(fld)
+DBASE_FIELD *fld;
+{
+  fread(fld->name, 1, 11, dbfile);
+  fread(&fld->type, 1, 1, dbfile);
+  /* "duh" believe me, it used to do just that before */
+  fread(&fld->data_ptr, 4, 1, dbfile);
+  fread(&fld->length, 1, 1, dbfile);
+  fread(&fld->dec_point, 1, 1, dbfile);
+  fread(fld->fill, 1, 14, dbfile);
+}
+
 /******************************************************
                                          db3_read_dic()
 This function is called with a file name to
@@ -159,30 +235,30 @@ read to create a record type to support the
 dbase file
 ******************************************************/
  
-void
+int
 db3_read_dic(flags)
 int	flags;
 {
     int             fields;
     DBASE_FIELD     *fld;
 
-    if(dbfile==-1) {
+    if(dbfile==NULL) {
         printf("open failed");
-        return;
+        return FALSE;
         }
-    read(dbfile,&dbhead,sizeof(DBASE_HEAD));
+    db3_read_head(&dbhead);
     if( !(dbhead.version==3 || dbhead.version==0x83) ) {
         printf ("Version %d not supported\n",dbhead.version);
 	if(dbhead.version==0x8b ) {
 	    printf ("dBase IV - partially known...\n");
 	}
-	return;
+	return FALSE;
     }
 
     if (flags & DB_FL_INFO) {
 	printf("File version  : %d\n",dbhead.version);
-	printf("Last update   : %02d/%02d/%2d\n", dbhead.l_update[1],dbhead.l_update[2],dbhead.l_update[0]);
-	printf("Number of recs: %ld\n",dbhead.count);
+	printf("Last update   : %02d/%02d/%2d\n", dbhead.l_update[1],dbhead.l_update[2],dbhead.l_update[0]+1900);
+	printf("Number of recs: %d\n",dbhead.count);
 	printf("Header length : %d\n",dbhead.header);
 	printf("Record length : %d\n",dbhead.lrecl);
     }
@@ -198,9 +274,9 @@ int	flags;
 	fld=(DBASE_FIELD *)malloc(sizeof(DBASE_FIELD));
 	if (!fld) {
 	    printf ("Not enough memory\n");
-	    return;
+	    return FALSE;
 	}
-	read(dbfile,fld,sizeof(DBASE_FIELD));
+	db3_read_field(fld);
 	if (! (flags & DB_FL_RESERVE))
 	    db3_cvt_fld (fld);
 	if (flags & DB_FL_DESCR) {
@@ -209,10 +285,10 @@ int	flags;
 	}
 	stack_field(fld);
     }
-    read(dbfile,Buffer,1);              /* read the silly little \r 0x0d character */
-    read(dbfile,Buffer,1);              /* strange, it only works if we read another byte */
+    /* Jump to where the data starts */
+    fseek(dbfile, dbhead.header, SEEK_SET);
 
-    return;
+    return TRUE;
 }
  
 /******************************************************
@@ -229,12 +305,12 @@ char	delim;
     int     bytes;
  
     while(cnt) {
-        bytes=read(dbfile,Buffer,dbhead.lrecl);
+        bytes=fread(Buffer,1,dbhead.lrecl,dbfile);
         if(bytes!=dbhead.lrecl)
             break;
 	/* Check if deleted == '*' */
-        if(Buffer[0]==' ') {
-            db3_print(flags, delim);
+	if(flags & DB_FL_DELETED || Buffer[0]==' ') {
+            db3_print(flags, delim, Buffer[0]);
             cnt--;
             }
         }
@@ -248,13 +324,17 @@ Print a single record
 ******************************************************/
  
 void
-db3_print(flags, delim)
+db3_print(flags, delim, deleted)
 int	flags;
 char	delim;
+char    deleted;
 {
     FLD_LIST    *temp;
  
     temp=db_fld_root;
+    if ((flags & DB_FL_DELETED) && (flags & DB_FL_BROWSE))
+      printf("%c%c", deleted, delim);
+
     while (temp) {
         memcpy(buf_work,temp->data,temp->fld->length);
         buf_work[temp->fld->length] = '\0';
@@ -267,6 +347,10 @@ char	delim;
 	    printf("%-11s: %s\n",temp->fld->name,db3_rm_tr_spc(db3_rm_ld_spc(buf_work)));
         temp=temp->next;
         }
+
+    if ((flags & DB_FL_DELETED) && ((flags & DB_FL_BROWSE) == 0) &&  (deleted != ' '))
+      printf("*Deleted*  : %c\n", deleted);
+
     printf("\n");
     return;
 }
@@ -289,7 +373,7 @@ DBASE_FIELD *fld;
     }
     list->fld=fld;
     if(!db_fld_root) {
-        list->data=Buffer+1;                            /*skip delete byte*/
+        list->data = (uint8_t *)Buffer+1;                   /*skip delete byte*/
         db_fld_root=list;
         return;
         }
