@@ -1,6 +1,6 @@
 /*
     dbview.c - View dBase III files
-    Copyright (c) 1995  Martin Schulze <Martin.Schulze@Infodrom.North.DE>
+    Copyright (c) 1995,96,2003  Martin Schulze <joey@infodrom.org>
 
     This file is part of the dbview package, a viewer for dBase II files.
 
@@ -15,8 +15,9 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    along with this program;  if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
+
 */
 
 /*
@@ -30,13 +31,15 @@
 #include "db_dump.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <getopt.h>
+#include <string.h>
 
 void help_short()
 {
     printf ("%s %s - %s, (c) 1996 by Martin Schulze\n", progname, version, longname);
     printf ("\n");
-    printf ("%s [-b [-t]] [-d delim] [-e] [-h] [-i] [-o] [-o] [-v] dbfile\n", progname);
+    printf ("%s [-b [-t]] [-d delim] [-e] [-h] [-i] [-D] [-o] [-v] dbfile\n", progname);
 }
 
 void help_long()
@@ -48,16 +51,17 @@ void help_long()
     printf ("  --description, -e      display field description\n");
     printf ("  --help, -h             display help\n");
     printf ("  --info, -i             display db information\n");
+    printf ("  --deleted, -D          display deleted records\n");
     printf ("  --omit, -o             omit db records\n");
     printf ("  --reserve, -r          reserve fieldnames from beeing translated\n");
     printf ("  --trim, -t             trim browse fields\n");
     printf ("  --version, -v          display version\n");
 }
 
-void main (int argc, char **argv)
+int main (int argc, char **argv)
 {
     int opt_index;
-    char c;
+    int c;
     static const struct option long_options[] =
       {
 	{"browse", no_argument, 0, 'b'},
@@ -66,6 +70,7 @@ void main (int argc, char **argv)
 	{"help", no_argument, 0, 'H'},
 	{"info", no_argument, 0, 'i'},
 	{"omit", no_argument, 0, 'o'},
+	{"deleted", no_argument, 0, 'D'},
 	{"reserve", no_argument, 0, 'r'},
 	{"trim", no_argument, 0, 't'},
 	{"version", no_argument, 0, 'v'},
@@ -76,7 +81,7 @@ void main (int argc, char **argv)
     char delim = ':';
 
     optind = 0;
-    while ( (c = getopt_long(argc, argv, "Hbd:ehiortv", long_options, &opt_index)) != -1 ) {
+    while ( (c = getopt_long(argc, argv, "Hbd:ehiDortv", long_options, &opt_index)) != -1 ) {
 	switch (c) {
 	case 'H':	/* --help */
 	    help_long (); exit (0);
@@ -98,6 +103,9 @@ void main (int argc, char **argv)
 	case 'i':
 	    flags |= DB_FL_INFO;
 	    break;
+	case 'D':
+	    flags |= DB_FL_DELETED;
+	    break;
 	case 'o':
 	    flags |= DB_FL_OMIT;
 	    break;
@@ -121,6 +129,5 @@ void main (int argc, char **argv)
     strcpy(dbfile, argv[optind]);
     /* if ! -f dbfile thnen -f dbfile.dbf */
 
-    db3_process (dbfile, flags, delim);
-
+    exit (db3_process (dbfile, flags, delim));
 }
